@@ -723,8 +723,6 @@ decode_window(svn_txdelta_window_t *window, svn_filesize_t sview_offset,
       svn_stringbuf_t *instout = svn_stringbuf_create_empty(pool);
       svn_stringbuf_t *ndout = svn_stringbuf_create_empty(pool);
 
-      /* these may in fact simply return references to insend */
-
       SVN_ERR(zlib_decode(insend, newlen, ndout,
                           SVN_DELTA_WINDOW_SIZE));
       SVN_ERR(zlib_decode(data, insend - data, instout,
@@ -739,7 +737,13 @@ decode_window(svn_txdelta_window_t *window, svn_filesize_t sview_offset,
     }
   else
     {
-      new_data->data = (const char *) insend;
+      /* Copy the data because an svn_string_t must have the invariant
+         data[len]=='\0'. */
+      char *buf = apr_palloc(pool, newlen + 1);
+
+      memcpy(buf, insend, newlen);
+      buf[newlen] = '\0';
+      new_data->data = buf;
       new_data->len = newlen;
     }
 
@@ -826,23 +830,23 @@ write_handler(void *baton,
 
       p = decode_file_offset(&sview_offset, p, end);
       if (p == NULL)
-        return SVN_NO_ERROR;
+        break;
 
       p = decode_size(&sview_len, p, end);
       if (p == NULL)
-        return SVN_NO_ERROR;
+        break;
 
       p = decode_size(&tview_len, p, end);
       if (p == NULL)
-        return SVN_NO_ERROR;
+        break;
 
       p = decode_size(&inslen, p, end);
       if (p == NULL)
-        return SVN_NO_ERROR;
+        break;
 
       p = decode_size(&newlen, p, end);
       if (p == NULL)
-        return SVN_NO_ERROR;
+        break;
 
       if (tview_len > SVN_DELTA_WINDOW_SIZE ||
           sview_len > SVN_DELTA_WINDOW_SIZE ||
@@ -900,7 +904,15 @@ write_handler(void *baton,
       db->subpool = newpool;
     }
 
-  /* NOTREACHED */
+  /* At this point we processed all integral windows and DB->BUFFER is empty
+     or contains partially read window header.
+     Check that unprocessed data is not larger that theoretical maximum
+     window header size. */
+  if (db->buffer->len > 5 * MAX_ENCODED_INT_LEN)
+    return svn_error_create(SVN_ERR_SVNDIFF_CORRUPT_WINDOW, NULL,
+                            _("Svndiff contains a too-large window header"));
+
+  return SVN_NO_ERROR;
 }
 
 /* Minimal svn_stream_t write handler, doing nothing */
