@@ -3,36 +3,48 @@
 #  svnsync_tests.py:  Tests SVNSync's repository mirroring capabilities.
 #
 #  Subversion is a tool for revision control.
-#  See http://subversion.tigris.org for more information.
+#  See http://subversion.apache.org for more information.
 #
 # ====================================================================
-# Copyright (c) 2005-2007 CollabNet.  All rights reserved.
+#    Licensed to the Apache Software Foundation (ASF) under one
+#    or more contributor license agreements.  See the NOTICE file
+#    distributed with this work for additional information
+#    regarding copyright ownership.  The ASF licenses this file
+#    to you under the Apache License, Version 2.0 (the
+#    "License"); you may not use this file except in compliance
+#    with the License.  You may obtain a copy of the License at
 #
-# This software is licensed as described in the file COPYING, which
-# you should have received as part of this distribution.  The terms
-# are also available at http://subversion.tigris.org/license-1.html.
-# If newer versions of this license are posted there, you may use a
-# newer version instead, at your option.
+#      http://www.apache.org/licenses/LICENSE-2.0
 #
+#    Unless required by applicable law or agreed to in writing,
+#    software distributed under the License is distributed on an
+#    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+#    KIND, either express or implied.  See the License for the
+#    specific language governing permissions and limitations
+#    under the License.
 ######################################################################
 
 # General modules
 import sys, os
 
+# Test suite-specific modules
+import re, urllib
+
 # Our testing module
 import svntest
 from svntest.verify import SVNUnexpectedStdout, SVNUnexpectedStderr
 from svntest.verify import SVNExpectedStderr
-from svntest.main import write_restrictive_svnserve_conf
 from svntest.main import server_has_partial_replay
 
 # (abbreviation)
-Skip = svntest.testcase.Skip
-SkipUnless = svntest.testcase.SkipUnless
-XFail = svntest.testcase.XFail
+Skip = svntest.testcase.Skip_deco
+SkipUnless = svntest.testcase.SkipUnless_deco
+XFail = svntest.testcase.XFail_deco
+Issues = svntest.testcase.Issues_deco
+Issue = svntest.testcase.Issue_deco
+Wimp = svntest.testcase.Wimp_deco
 Item = svntest.wc.StateItem
 
-
 ######################################################################
 # Helper routines
 
@@ -47,12 +59,25 @@ def build_repos(sbox):
   svntest.main.create_repos(sbox.repo_dir)
 
 
-def run_sync(url, expected_error=None):
+def run_sync(url, source_url=None, expected_error=None,
+             source_prop_encoding=None):
   "Synchronize the mirror repository with the master"
-  exit_code, output, errput = svntest.main.run_svnsync(
-    "synchronize", url,
-    "--username", svntest.main.wc_author,
-    "--password", svntest.main.wc_passwd)
+  if source_url is not None:
+    args = ["synchronize", url, source_url,
+      "--username", svntest.main.wc_author,
+      "--password", svntest.main.wc_passwd]
+  else: # Allow testing of old source-URL-less syntax
+    args = ["synchronize", url,
+      "--username", svntest.main.wc_author,
+      "--password", svntest.main.wc_passwd]
+  if source_prop_encoding:
+    args.append("--source-prop-encoding")
+    args.append(source_prop_encoding)
+
+  exit_code, output, errput = svntest.main.run_svnsync(*args)
+  for index, line in enumerate(errput[:]):
+    if re.search("warning: W200007", line):
+      del errput[index]
   if errput:
     if expected_error is None:
       raise SVNUnexpectedStderr(errput)
@@ -67,12 +92,20 @@ def run_sync(url, expected_error=None):
     # should be: ['Committed revision 1.\n', 'Committed revision 2.\n']
     raise SVNUnexpectedStdout("Missing stdout")
 
-def run_copy_revprops(url, expected_error=None):
+def run_copy_revprops(url, source_url, expected_error=None,
+                      source_prop_encoding=None):
   "Copy revprops to the mirror repository from the master"
-  exit_code, output, errput = svntest.main.run_svnsync(
-    "copy-revprops", url,
+  args = ["copy-revprops", url, source_url,
     "--username", svntest.main.wc_author,
-    "--password", svntest.main.wc_passwd)
+    "--password", svntest.main.wc_passwd]
+  if source_prop_encoding:
+    args.append("--source-prop-encoding")
+    args.append(source_prop_encoding)
+
+  exit_code, output, errput = svntest.main.run_svnsync(*args)
+  for index, line in enumerate(errput[:]):
+    if re.search("warning: W200007", line):
+      del errput[index]
   if errput:
     if expected_error is None:
       raise SVNUnexpectedStderr(errput)
@@ -88,12 +121,19 @@ def run_copy_revprops(url, expected_error=None):
     #             'Copied properties for revision 2.\n']
     raise SVNUnexpectedStdout("Missing stdout")
 
-def run_init(dst_url, src_url):
+def run_init(dst_url, src_url, source_prop_encoding=None):
   "Initialize the mirror repository from the master"
-  exit_code, output, errput = svntest.main.run_svnsync(
-    "initialize", dst_url, src_url,
+  args = ["initialize", dst_url, src_url,
     "--username", svntest.main.wc_author,
-    "--password", svntest.main.wc_passwd)
+    "--password", svntest.main.wc_passwd]
+  if source_prop_encoding:
+    args.append("--source-prop-encoding")
+    args.append(source_prop_encoding)
+
+  exit_code, output, errput = svntest.main.run_svnsync(*args)
+  for index, line in enumerate(errput[:]):
+    if re.search("warning: W200007", line):
+      del errput[index]
   if errput:
     raise SVNUnexpectedStderr(errput)
   if output != ['Copied properties for revision 0.\n']:
@@ -122,20 +162,17 @@ def run_info(url, expected_error=None):
     raise SVNUnexpectedStdout("Missing stdout")
 
 
-def run_test(sbox, dump_file_name, subdir = None, exp_dump_file_name = None):
-  """Load a dump file, sync repositories, and compare contents with the original
-or another dump file."""
+def setup_and_sync(sbox, dump_file_contents, subdir=None,
+                   bypass_prop_validation=False, source_prop_encoding=None,
+                   is_src_ra_local=None, is_dest_ra_local=None):
+  """Create a repository for SBOX, load it with DUMP_FILE_CONTENTS, then create a mirror repository and sync it with SBOX. If is_src_ra_local or is_dest_ra_local is True, then run_init, run_sync, and run_copy_revprops will use the file:// scheme for the source and destination URLs.  Return the mirror sandbox."""
 
   # Create the empty master repository.
   build_repos(sbox)
 
-  # This directory contains all the dump files
-  svnsync_tests_dir = os.path.join(os.path.dirname(sys.argv[0]),
-                                   'svnsync_tests_data')
-  # Load the specified dump file into the master repository.
-  master_dumpfile_contents = file(os.path.join(svnsync_tests_dir,
-                                               dump_file_name)).readlines()
-  svntest.actions.run_and_verify_load(sbox.repo_dir, master_dumpfile_contents)
+  # Load the repository from DUMP_FILE_PATH.
+  svntest.actions.run_and_verify_load(sbox.repo_dir, dump_file_contents,
+                                      bypass_prop_validation)
 
   # Create the empty destination repository.
   dest_sbox = sbox.clone_dependent()
@@ -151,12 +188,33 @@ or another dump file."""
   svntest.actions.enable_revprop_changes(dest_sbox.repo_dir)
 
   repo_url = sbox.repo_url
+  cwd = os.getcwd()
+  if is_src_ra_local:
+    repo_url = svntest.main.file_scheme_prefix + \
+                        urllib.pathname2url(os.path.join(cwd, sbox.repo_dir))
+
   if subdir:
     repo_url = repo_url + subdir
-  run_init(dest_sbox.repo_url, repo_url)
 
-  run_sync(dest_sbox.repo_url)
-  run_copy_revprops(dest_sbox.repo_url)
+  dest_repo_url = dest_sbox.repo_url
+  if is_dest_ra_local:
+    dest_repo_url = svntest.main.file_scheme_prefix + \
+                    urllib.pathname2url(os.path.join(cwd, dest_sbox.repo_dir))
+  run_init(dest_repo_url, repo_url, source_prop_encoding)
+
+  run_sync(dest_repo_url, repo_url,
+           source_prop_encoding=source_prop_encoding)
+  run_copy_revprops(dest_repo_url, repo_url,
+                    source_prop_encoding=source_prop_encoding)
+
+  return dest_sbox
+
+def verify_mirror(dest_sbox, exp_dump_file_contents):
+  """Compare the contents of the mirror repository in DEST_SBOX with
+     EXP_DUMP_FILE_CONTENTS, by comparing the parsed dump stream content.
+
+     First remove svnsync rev-props from the DEST_SBOX repository.
+  """
 
   # Remove some SVNSync-specific housekeeping properties from the
   # mirror repository in preparation for the comparison dump.
@@ -169,17 +227,40 @@ or another dump file."""
   # Create a dump file from the mirror repository.
   dest_dump = svntest.actions.run_and_verify_dump(dest_sbox.repo_dir)
 
+  svntest.verify.compare_dump_files(
+    "Dump files", "DUMP", exp_dump_file_contents, dest_dump)
+
+def run_test(sbox, dump_file_name, subdir=None, exp_dump_file_name=None,
+             bypass_prop_validation=False, source_prop_encoding=None,
+             is_src_ra_local=None, is_dest_ra_local=None):
+
+  """Load a dump file, sync repositories, and compare contents with the original
+or another dump file."""
+
+  # This directory contains all the dump files
+  svnsync_tests_dir = os.path.join(os.path.dirname(sys.argv[0]),
+                                   'svnsync_tests_data')
+
+  # Load the specified dump file into the master repository.
+  master_dumpfile_contents = open(os.path.join(svnsync_tests_dir,
+                                               dump_file_name),
+                                  'rb').readlines()
+
+  dest_sbox = setup_and_sync(sbox, master_dumpfile_contents, subdir,
+                             bypass_prop_validation, source_prop_encoding,
+                             is_src_ra_local, is_dest_ra_local)
+
   # Compare the dump produced by the mirror repository with either the original
   # dump file (used to create the master repository) or another specified dump
   # file.
   if exp_dump_file_name:
-    exp_master_dumpfile_contents = file(os.path.join(svnsync_tests_dir,
-                                        exp_dump_file_name)).readlines()
+    exp_dump_file_contents = open(os.path.join(svnsync_tests_dir,
+                                  exp_dump_file_name), 'rb').readlines()
   else:
-    exp_master_dumpfile_contents = master_dumpfile_contents
+    exp_dump_file_contents = master_dumpfile_contents
+
+  verify_mirror(dest_sbox, exp_dump_file_contents)
 
-  svntest.verify.compare_and_display_lines(
-    "Dump files", "DUMP", exp_master_dumpfile_contents, dest_dump)
 
 
 ######################################################################
@@ -243,7 +324,7 @@ def tag_with_modified_file(sbox):
 
 def dir_prop_change(sbox):
   "directory property changes"
-  run_test(sbox, "dir_prop_change.dump")
+  run_test(sbox, "dir-prop-change.dump")
 
 #----------------------------------------------------------------------
 
@@ -299,377 +380,9 @@ def detect_meddling(sbox):
                                      '-m', 'msg',
                                      dest_sbox.wc_dir)
 
-  run_sync(dest_sbox.repo_url,
+  run_sync(dest_sbox.repo_url, None,
            ".*Destination HEAD \\(2\\) is not the last merged revision \\(1\\).*")
 
-#----------------------------------------------------------------------
-
-def basic_authz(sbox):
-  "verify that unreadable content is not synced"
-
-  sbox.build("svnsync-basic-authz")
-
-  write_restrictive_svnserve_conf(sbox.repo_dir)
-
-  dest_sbox = sbox.clone_dependent()
-  build_repos(dest_sbox)
-
-  svntest.actions.enable_revprop_changes(dest_sbox.repo_dir)
-
-  run_init(dest_sbox.repo_url, sbox.repo_url)
-
-  svntest.main.file_write(sbox.authz_file,
-                          "[svnsync-basic-authz:/]\n"
-                          "* = r\n"
-                          "\n"
-                          "[svnsync-basic-authz:/A/B]\n"
-                          "* = \n"
-                          "\n"
-                          "[svnsync-basic-authz-1:/]\n"
-                          "* = rw\n")
-
-  run_sync(dest_sbox.repo_url)
-
-  lambda_url = dest_sbox.repo_url + '/A/B/lambda'
-
-  # this file should have been blocked by authz
-  svntest.actions.run_and_verify_svn(None,
-                                     [], svntest.verify.AnyOutput,
-                                     'cat',
-                                     lambda_url)
-
-#----------------------------------------------------------------------
-
-def copy_from_unreadable_dir(sbox):
-  "verify that copies from unreadable dirs work"
-
-  sbox.build("svnsync-copy-from-unreadable-dir")
-
-  B_url = sbox.repo_url + '/A/B'
-  P_url = sbox.repo_url + '/A/P'
-
-  # Set a property on the directory we're going to copy, and a file in it, to
-  # confirm that they're transmitted when we later sync the copied directory
-  svntest.actions.run_and_verify_svn(None,
-                                     None,
-                                     [],
-                                     'pset',
-                                     'foo',
-                                     'bar',
-                                     sbox.wc_dir + '/A/B/lambda')
-
-  svntest.actions.run_and_verify_svn(None,
-                                     None,
-                                     [],
-                                     'pset',
-                                     'baz',
-                                     'zot',
-                                     sbox.wc_dir + '/A/B')
-
-  svntest.actions.run_and_verify_svn(None,
-                                     None,
-                                     [],
-                                     'ci',
-                                     sbox.wc_dir + '/A/B',
-                                     '-m', 'log_msg')
-
-  # Now copy that directory so we'll see it in our synced copy
-  svntest.actions.run_and_verify_svn(None,
-                                     None,
-                                     [],
-                                     'cp',
-                                     B_url,
-                                     P_url,
-                                     '-m', 'Copy B to P')
-
-  write_restrictive_svnserve_conf(sbox.repo_dir)
-
-  dest_sbox = sbox.clone_dependent()
-  build_repos(dest_sbox)
-
-  svntest.actions.enable_revprop_changes(dest_sbox.repo_dir)
-
-  fp = open(sbox.authz_file, 'w')
-
-  # For mod_dav_svn's parent path setup we need per-repos permissions in
-  # the authz file...
-  if sbox.repo_url.startswith('http'):
-    fp.write("[svnsync-copy-from-unreadable-dir:/]\n" +
-             "* = r\n" +
-             "\n" +
-             "[svnsync-copy-from-unreadable-dir:/A/B]\n" +
-             "* = \n" +
-             "\n" +
-             "[svnsync-copy-from-unreadable-dir-1:/]\n" +
-             "* = rw")
-
-  # Otherwise we can just go with the permissions needed for the source
-  # repository.
-  else:
-    fp.write("[/]\n" +
-             "* = r\n" +
-             "\n" +
-             "[/A/B]\n" +
-             "* =\n")
-  fp.close()
-
-  run_init(dest_sbox.repo_url, sbox.repo_url)
-
-  run_sync(dest_sbox.repo_url)
-
-  expected_out = [
-    'Changed paths:\n',
-    '   A /A/P\n',
-    '   A /A/P/E\n',
-    '   A /A/P/E/alpha\n',
-    '   A /A/P/E/beta\n',
-    '   A /A/P/F\n',
-    '   A /A/P/lambda\n',
-    '\n',
-    '\n', # log message is stripped
-  ]
-
-  exit_code, out, err = svntest.main.run_svn(None,
-                                             'log',
-                                             '-r', '3',
-                                             '-v',
-                                             dest_sbox.repo_url)
-
-  if err:
-    raise SVNUnexpectedStderr(err)
-
-  svntest.verify.compare_and_display_lines(None,
-                                           'LOG',
-                                           expected_out,
-                                           out[2:11])
-
-  svntest.actions.run_and_verify_svn(None,
-                                     ['bar\n'],
-                                     [],
-                                     'pget',
-                                     'foo',
-                                     dest_sbox.repo_url + '/A/P/lambda')
-
-  svntest.actions.run_and_verify_svn(None,
-                                     ['zot\n'],
-                                     [],
-                                     'pget',
-                                     'baz',
-                                     dest_sbox.repo_url + '/A/P')
-
-# Issue 2705.
-def copy_with_mod_from_unreadable_dir(sbox):
-  "verify copies with mods from unreadable dirs"
-
-  sbox.build("svnsync-copy-with-mod-from-unreadable-dir")
-
-  # Make a copy of the B directory.
-  svntest.actions.run_and_verify_svn(None,
-                                     None,
-                                     [],
-                                     'cp',
-                                     sbox.wc_dir + '/A/B',
-                                     sbox.wc_dir + '/A/P')
-
-  # Set a property inside the copied directory.
-  svntest.actions.run_and_verify_svn(None,
-                                     None,
-                                     [],
-                                     'pset',
-                                     'foo',
-                                     'bar',
-                                     sbox.wc_dir + '/A/P/lambda')
-
-  # Add a new directory and file inside the copied directory.
-  svntest.actions.run_and_verify_svn(None,
-                                     None,
-                                     [],
-                                     'mkdir',
-                                     sbox.wc_dir + '/A/P/NEW-DIR')
-
-  svntest.main.file_append(sbox.wc_dir + '/A/P/E/new-file', "bla bla")
-  svntest.main.run_svn(None, 'add', sbox.wc_dir + '/A/P/E/new-file')
-
-  # Delete a file inside the copied directory.
-  svntest.actions.run_and_verify_svn(None,
-                                     None,
-                                     [],
-                                     'rm',
-                                     sbox.wc_dir + '/A/P/E/beta')
-
-  # Commit the copy-with-modification.
-  svntest.actions.run_and_verify_svn(None,
-                                     None,
-                                     [],
-                                     'ci',
-                                     sbox.wc_dir,
-                                     '-m', 'log_msg')
-
-  # Lock down the source repository.
-  write_restrictive_svnserve_conf(sbox.repo_dir)
-
-  dest_sbox = sbox.clone_dependent()
-  build_repos(dest_sbox)
-
-  svntest.actions.enable_revprop_changes(dest_sbox.repo_dir)
-
-  fp = open(sbox.authz_file, 'w')
-
-  # For mod_dav_svn's parent path setup we need per-repos permissions in
-  # the authz file...
-  if sbox.repo_url.startswith('http'):
-    fp.write("[svnsync-copy-with-mod-from-unreadable-dir:/]\n" +
-             "* = r\n" +
-             "\n" +
-             "[svnsync-copy-with-mod-from-unreadable-dir:/A/B]\n" +
-             "* = \n" +
-             "\n" +
-             "[svnsync-copy-with-mod-from-unreadable-dir-1:/]\n" +
-             "* = rw")
-
-  # Otherwise we can just go with the permissions needed for the source
-  # repository.
-  else:
-    fp.write("[/]\n" +
-             "* = r\n" +
-             "\n" +
-             "[/A/B]\n" +
-             "* =\n")
-  fp.close()
-
-  run_init(dest_sbox.repo_url, sbox.repo_url)
-
-  run_sync(dest_sbox.repo_url)
-
-  expected_out = [
-    'Changed paths:\n',
-    '   A /A/P\n',
-    '   A /A/P/E\n',
-    '   A /A/P/E/alpha\n',
-    '   A /A/P/E/new-file\n',
-    '   A /A/P/F\n',
-    '   A /A/P/NEW-DIR\n',
-    '   A /A/P/lambda\n',
-    '\n',
-    '\n', # log message is stripped
-  ]
-
-  exit_code, out, err = svntest.main.run_svn(None,
-                                             'log',
-                                             '-r', '2',
-                                             '-v',
-                                             dest_sbox.repo_url)
-
-  if err:
-    raise SVNUnexpectedStderr(err)
-
-  svntest.verify.compare_and_display_lines(None,
-                                           'LOG',
-                                           expected_out,
-                                           out[2:12])
-
-  svntest.actions.run_and_verify_svn(None,
-                                     ['bar\n'],
-                                     [],
-                                     'pget',
-                                     'foo',
-                                     dest_sbox.repo_url + '/A/P/lambda')
-
-# Issue 2705.
-def copy_with_mod_from_unreadable_dir_and_copy(sbox):
-  "verify copies with mods from unreadable dirs +copy"
-
-  sbox.build("svnsync-copy-with-mod-from-unreadable-dir-and-copy")
-
-  # Make a copy of the B directory.
-  svntest.actions.run_and_verify_svn(None,
-                                     None,
-                                     [],
-                                     'cp',
-                                     sbox.wc_dir + '/A/B',
-                                     sbox.wc_dir + '/A/P')
-
-
-  # Copy a (readable) file into the copied directory.
-  svntest.actions.run_and_verify_svn(None,
-                                     None,
-                                     [],
-                                     'cp',
-                                     sbox.wc_dir + '/A/D/gamma',
-                                     sbox.wc_dir + '/A/P/E')
-
-
-  # Commit the copy-with-modification.
-  svntest.actions.run_and_verify_svn(None,
-                                     None,
-                                     [],
-                                     'ci',
-                                     sbox.wc_dir,
-                                     '-m', 'log_msg')
-
-  # Lock down the source repository.
-  write_restrictive_svnserve_conf(sbox.repo_dir)
-
-  dest_sbox = sbox.clone_dependent()
-  build_repos(dest_sbox)
-
-  svntest.actions.enable_revprop_changes(dest_sbox.repo_dir)
-
-  fp = open(sbox.authz_file, 'w')
-
-  # For mod_dav_svn's parent path setup we need per-repos permissions in
-  # the authz file...
-  if sbox.repo_url.startswith('http'):
-    fp.write("[svnsync-copy-with-mod-from-unreadable-dir-and-copy:/]\n" +
-             "* = r\n" +
-             "\n" +
-             "[svnsync-copy-with-mod-from-unreadable-dir-and-copy:/A/B]\n" +
-             "* = \n" +
-             "\n" +
-             "[svnsync-copy-with-mod-from-unreadable-dir-and-copy-1:/]\n" +
-             "* = rw")
-
-  # Otherwise we can just go with the permissions needed for the source
-  # repository.
-  else:
-    fp.write("[/]\n" +
-             "* = r\n" +
-             "\n" +
-             "[/A/B]\n" +
-             "* =\n")
-  fp.close()
-
-  run_init(dest_sbox.repo_url, sbox.repo_url)
-
-  run_sync(dest_sbox.repo_url)
-
-  expected_out = [
-    'Changed paths:\n',
-    '   A /A/P\n',
-    '   A /A/P/E\n',
-    '   A /A/P/E/alpha\n',
-    '   A /A/P/E/beta\n',
-    '   A /A/P/E/gamma (from /A/D/gamma:1)\n',
-    '   A /A/P/F\n',
-    '   A /A/P/lambda\n',
-    '\n',
-    '\n', # log message is stripped
-  ]
-
-  exit_code, out, err = svntest.main.run_svn(None,
-                                             'log',
-                                             '-r', '2',
-                                             '-v',
-                                             dest_sbox.repo_url)
-
-  if err:
-    raise SVNUnexpectedStderr(err)
-
-  svntest.verify.compare_and_display_lines(None,
-                                           'LOG',
-                                           expected_out,
-                                           out[2:12])
-
 def url_encoding(sbox):
   "test url encoding issues"
   run_test(sbox, "url-encoding-bug.dump")
@@ -686,17 +399,20 @@ def copy_revprops(sbox):
   "test copying revprops other than svn:*"
   run_test(sbox, "revprops.dump")
 
+@SkipUnless(server_has_partial_replay)
 def only_trunk(sbox):
   "test syncing subdirectories"
   run_test(sbox, "svnsync-trunk-only.dump", "/trunk",
            "svnsync-trunk-only.expected.dump")
 
+@SkipUnless(server_has_partial_replay)
 def only_trunk_A_with_changes(sbox):
   "test syncing subdirectories with changes on root"
   run_test(sbox, "svnsync-trunk-A-changes.dump", "/trunk/A",
            "svnsync-trunk-A-changes.expected.dump")
 
 # test for issue #2904
+@Issue(2904)
 def move_and_modify_in_the_same_revision(sbox):
   "test move parent and modify child file in same rev"
   run_test(sbox, "svnsync-move-and-modify.dump")
@@ -742,6 +458,194 @@ def info_not_synchronized(sbox):
   run_info(sbox.repo_url,
            ".*Repository '%s' is not initialized.*" % sbox.repo_url)
 
+#----------------------------------------------------------------------
+
+def copy_bad_line_endings(sbox):
+  "copy with inconsistent line endings in svn:* props"
+  run_test(sbox, "copy-bad-line-endings.dump",
+           exp_dump_file_name="copy-bad-line-endings.expected.dump",
+           bypass_prop_validation=True)
+
+def copy_bad_line_endings2(sbox):
+  "copy with non-LF line endings in svn:* props"
+  run_test(sbox, "copy-bad-line-endings2.dump",
+           exp_dump_file_name="copy-bad-line-endings2.expected.dump",
+           bypass_prop_validation=True)
+
+def copy_bad_encoding(sbox):
+  "copy and reencode non-UTF-8 svn:* props"
+  run_test(sbox, "copy-bad-encoding.dump",
+           exp_dump_file_name="copy-bad-encoding.expected.dump",
+           bypass_prop_validation=True, source_prop_encoding="ISO-8859-3")
+
+#----------------------------------------------------------------------
+
+def delete_svn_props(sbox):
+  "copy with svn:* prop deletions"
+  run_test(sbox, "delete-svn-props.dump")
+
+@Issue(3438)
+def commit_a_copy_of_root(sbox):
+  "commit a copy of root causes sync to fail"
+  #Testcase for issue 3438.
+  run_test(sbox, "repo-with-copy-of-root-dir.dump")
+
+
+# issue #3641 'svnsync fails to partially copy a repository'.
+# This currently fails because while replacements with history
+# within copies are handled, replacements without history inside
+# copies cause the sync to fail:
+#
+#   >svnsync synchronize %TEST_REPOS_ROOT_URL%/svnsync_tests-29-1
+#    %TEST_REPOS_ROOT_URL%/svnsync_tests-29/trunk/H
+#   Transmitting file data ...\..\..\subversion\svnsync\main.c:1444: (apr_err=160013)
+#   ..\..\..\subversion\svnsync\main.c:1391: (apr_err=160013)
+#   ..\..\..\subversion\libsvn_ra\ra_loader.c:1168: (apr_err=160013)
+#   ..\..\..\subversion\libsvn_delta\path_driver.c:254: (apr_err=160013)
+#   ..\..\..\subversion\libsvn_repos\replay.c:480: (apr_err=160013)
+#   ..\..\..\subversion\libsvn_repos\replay.c:276: (apr_err=160013)
+#   ..\..\..\subversion\libsvn_repos\replay.c:290: (apr_err=160013)
+#   ..\..\..\subversion\libsvn_fs_base\tree.c:1258: (apr_err=160013)
+#   ..\..\..\subversion\libsvn_fs_base\tree.c:1258: (apr_err=160013)
+#   ..\..\..\subversion\libsvn_fs_base\tree.c:1236: (apr_err=160013)
+#   ..\..\..\subversion\libsvn_fs_base\tree.c:931: (apr_err=160013)
+#   ..\..\..\subversion\libsvn_fs_base\tree.c:742: (apr_err=160013)
+#   svnsync: File not found: revision 4, path '/trunk/H/Z/B/lambda'
+#
+# See also http://svn.haxx.se/dev/archive-2010-11/0411.shtml and
+#
+#
+# Note: For those who may poke around this test in the future, r3 of
+# delete-revprops.dump was created with the following svnmucc command:
+#
+# svnmucc.exe -mm cp head %ROOT_URL%/trunk/A %ROOT_URL%/trunk/H
+#                 rm %ROOT_URL%/trunk/H/B
+#                 cp head %ROOT_URL%/trunk/X %ROOT_URL%/trunk/B
+#
+# r4 was created with this svnmucc command:
+#
+# svnmucc.exe -mm cp head %ROOT_URL%/trunk/A %ROOT_URL%/trunk/H/Z
+#                 rm %ROOT_URL%/trunk/H/Z/B
+#                 mkdir %ROOT_URL%/trunk/H/Z/B
+@Issue(3641)
+def descend_into_replace(sbox):
+  "descending into replaced dir looks in src"
+  run_test(sbox, "descend-into-replace.dump", subdir='/trunk/H',
+           exp_dump_file_name = "descend-into-replace.expected.dump")
+
+# issue #3728
+@Issue(3728)
+def delete_revprops(sbox):
+  "copy-revprops with removals"
+  svnsync_tests_dir = os.path.join(os.path.dirname(sys.argv[0]),
+                                   'svnsync_tests_data')
+  initial_contents  = open(os.path.join(svnsync_tests_dir,
+                                        "delete-revprops.dump"),
+                           'rb').readlines()
+  expected_contents = open(os.path.join(svnsync_tests_dir,
+                                        "delete-revprops.expected.dump"),
+                           'rb').readlines()
+
+  # Create the initial repos and mirror, and sync 'em.
+  dest_sbox = setup_and_sync(sbox, initial_contents)
+
+  # Now remove a revprop from r1 of the source, and run 'svnsync
+  # copy-revprops' to re-sync 'em.
+  svntest.actions.enable_revprop_changes(sbox.repo_dir)
+  exit_code, out, err = svntest.main.run_svn(None,
+                                             'pdel',
+                                             '-r', '1',
+                                             '--revprop',
+                                             'issue-id',
+                                             sbox.repo_url)
+  if err:
+    raise SVNUnexpectedStderr(err)
+  run_copy_revprops(dest_sbox.repo_url, sbox.repo_url)
+
+  # Does the result look as we expected?
+  verify_mirror(dest_sbox, expected_contents)
+
+@Issue(3870)
+@SkipUnless(svntest.main.is_posix_os)
+def fd_leak_sync_from_serf_to_local(sbox):
+  "fd leak during sync from serf to local"
+  import resource
+  resource.setrlimit(resource.RLIMIT_NOFILE, (128, 128))
+  run_test(sbox, "largemods.dump", is_src_ra_local=None, is_dest_ra_local=True)
+
+#----------------------------------------------------------------------
+
+@Issue(4476)
+def mergeinfo_contains_r0(sbox):
+  "mergeinfo contains r0"
+
+  def make_node_record(node_name, mi):
+    """Return a dumpfile node-record for adding a (directory) node named
+       NODE_NAME with mergeinfo MI. Return it as a list of newline-terminated
+       lines.
+    """
+    headers_tmpl = """\
+Node-path: %s
+Node-kind: dir
+Node-action: add
+Prop-content-length: %d
+Content-length: %d
+"""
+    content_tmpl = """\
+K 13
+svn:mergeinfo
+V %d
+%s
+PROPS-END
+"""
+    content = content_tmpl % (len(mi), mi)
+    headers = headers_tmpl % (node_name, len(content), len(content))
+    record = headers + '\n' + content + '\n\n'
+    return record.splitlines(True)
+
+  # The test case mergeinfo (before, after) syncing, separated here with
+  # spaces instead of newlines
+  test_mi = [
+    ("",            ""),  # unchanged
+    ("/a:1",        "/a:1"),
+    ("/a:1 /b:1*,2","/a:1 /b:1*,2"),
+    ("/:0:1",       "/:0:1"),  # unchanged; colon-zero in filename
+    ("/a:0",        ""),  # dropped entirely
+    ("/a:0*",       ""),
+    ("/a:0 /b:0*",  ""),
+    ("/a:1 /b:0",   "/a:1"),  # one kept, one dropped
+    ("/a:0 /b:1",   "/b:1"),
+    ("/a:0,1 /b:1", "/a:1 /b:1"),  # one kept, one changed
+    ("/a:1 /b:0,1", "/a:1 /b:1"),
+    ("/a:0,1 /b:0*,1 /c:0,2 /d:0-1 /e:0-1,3 /f:0-2 /g:0-3",
+     "/a:1 /b:1 /c:2 /d:1 /e:1,3 /f:1-2 /g:1-3"),  # all changed
+    ("/a:0:0-1",    "/a:0:1"),  # changed; colon-zero in filename
+    ]
+
+  # Get the constant prefix for each dumpfile
+  dump_file_name = "mergeinfo-contains-r0.dump"
+  svnsync_tests_dir = os.path.join(os.path.dirname(sys.argv[0]),
+                                   'svnsync_tests_data')
+  dump_in = open(os.path.join(svnsync_tests_dir, dump_file_name),
+                 'rb').readlines()
+  dump_out = list(dump_in)  # duplicate the list
+
+  # Add dumpfile node records containing the test mergeinfo
+  for n, mi in enumerate(test_mi):
+    node_name = "D" + str(n)
+
+    mi_in = mi[0].replace(' ', '\n')
+    mi_out = mi[1].replace(' ', '\n')
+    dump_in.extend(make_node_record(node_name, mi_in))
+    dump_out.extend(make_node_record(node_name, mi_out))
+
+  # Run the sync
+  dest_sbox = setup_and_sync(sbox, dump_in, bypass_prop_validation=True)
+
+  # Compare the dump produced by the mirror repository with expected
+  verify_mirror(dest_sbox, dump_out)
+
+
 ########################################################################
 # Run the tests
 
@@ -754,33 +658,34 @@ test_list = [ None,
               modified_in_place,
               tag_empty_trunk,
               tag_trunk_with_dir,
-              tag_trunk_with_file2,
               tag_trunk_with_file,
+              tag_trunk_with_file2,
               tag_with_modified_file,
               dir_prop_change,
               file_dir_file,
               copy_parent_modify_prop,
               detect_meddling,
-              Skip(basic_authz, svntest.main.is_ra_type_file),
-              Skip(copy_from_unreadable_dir, svntest.main.is_ra_type_file),
-              Skip(copy_with_mod_from_unreadable_dir,
-                   svntest.main.is_ra_type_file),
-              Skip(copy_with_mod_from_unreadable_dir_and_copy,
-                   svntest.main.is_ra_type_file),
               url_encoding,
               no_author,
               copy_revprops,
-              SkipUnless(only_trunk,
-                         server_has_partial_replay),
-              SkipUnless(only_trunk_A_with_changes,
-                         server_has_partial_replay),
+              only_trunk,
+              only_trunk_A_with_changes,
               move_and_modify_in_the_same_revision,
               info_synchronized,
               info_not_synchronized,
+              copy_bad_line_endings,
+              copy_bad_line_endings2,
+              copy_bad_encoding,
+              delete_svn_props,
+              commit_a_copy_of_root,
+              descend_into_replace,
+              delete_revprops,
+              fd_leak_sync_from_serf_to_local, # calls setrlimit
+              mergeinfo_contains_r0,
              ]
 
 if __name__ == '__main__':
-  svntest.main.run_tests(test_list, serial_only = True)
+  svntest.main.run_tests(test_list)
   # NOTREACHED
 
 
