@@ -1,17 +1,22 @@
 /* strings-reps-test.c --- test `strings' and `representations' interfaces
  *
  * ====================================================================
- * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
+ *    Licensed to the Apache Software Foundation (ASF) under one
+ *    or more contributor license agreements.  See the NOTICE file
+ *    distributed with this work for additional information
+ *    regarding copyright ownership.  The ASF licenses this file
+ *    to you under the Apache License, Version 2.0 (the
+ *    "License"); you may not use this file except in compliance
+ *    with the License.  You may obtain a copy of the License at
  *
- * This software is licensed as described in the file COPYING, which
- * you should have received as part of this distribution.  The terms
- * are also available at http://subversion.tigris.org/license-1.html.
- * If newer versions of this license are posted there, you may use a
- * newer version instead, at your option.
+ *      http://www.apache.org/licenses/LICENSE-2.0
  *
- * This software consists of voluntary contributions made by many
- * individuals.  For exact contribution history, see the revision
- * history and logs, available at http://subversion.tigris.org/.
+ *    Unless required by applicable law or agreed to in writing,
+ *    software distributed under the License is distributed on an
+ *    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ *    KIND, either express or implied.  See the License for the
+ *    specific language governing permissions and limitations
+ *    under the License.
  * ====================================================================
  */
 
@@ -91,20 +96,13 @@ txn_body_delete_rep(void *baton, trail_t *trail)
 /* Representation Table Test functions. */
 
 static svn_error_t *
-write_new_rep(const char **msg,
-              svn_boolean_t msg_only,
-              svn_test_opts_t *opts,
+write_new_rep(const svn_test_opts_t *opts,
               apr_pool_t *pool)
 {
   struct rep_args args;
   const char *rep = "((fulltext 0 ) a83t2Z0q)";
   svn_fs_t *fs;
 
-  *msg = "write a new rep, get a new key back";
-
-  if (msg_only)
-    return SVN_NO_ERROR;
-
   /* Create a new fs and repos */
   SVN_ERR(svn_test__create_bdb_fs
           (&fs, "test-repo-write-new-rep", opts,
@@ -117,7 +115,7 @@ write_new_rep(const char **msg,
 
   /* Write new rep to reps table. */
   SVN_ERR(svn_fs_base__retry_txn(args.fs, txn_body_write_new_rep, &args,
-                                 pool));
+                                 FALSE, pool));
 
   if (args.key == NULL)
     return svn_error_create(SVN_ERR_FS_GENERAL, NULL,
@@ -128,9 +126,7 @@ write_new_rep(const char **msg,
 
 
 static svn_error_t *
-write_rep(const char **msg,
-          svn_boolean_t msg_only,
-          svn_test_opts_t *opts,
+write_rep(const svn_test_opts_t *opts,
           apr_pool_t *pool)
 {
   struct rep_args new_args;
@@ -139,11 +135,6 @@ write_rep(const char **msg,
   const char *rep = "((fulltext 0 ) kfogel31337)";
   svn_fs_t *fs;
 
-  *msg = "write a new rep, then overwrite it";
-
-  if (msg_only)
-    return SVN_NO_ERROR;
-
   /* Create a new fs and repos */
   SVN_ERR(svn_test__create_bdb_fs
           (&fs, "test-repo-write-rep", opts,
@@ -155,8 +146,8 @@ write_rep(const char **msg,
   new_args.key = NULL;
 
   /* Write new rep to reps table. */
-  SVN_ERR(svn_fs_base__retry_txn(new_args.fs,
-                                 txn_body_write_new_rep, &new_args, pool));
+  SVN_ERR(svn_fs_base__retry_txn(new_args.fs, txn_body_write_new_rep,
+                                 &new_args, FALSE, pool));
 
   /* Make sure we got a valid key. */
   if (new_args.key == NULL)
@@ -169,17 +160,15 @@ write_rep(const char **msg,
   args.key = new_args.key;
 
   /* Overwrite first rep in reps table. */
-  SVN_ERR(svn_fs_base__retry_txn(new_args.fs,
-                                 txn_body_write_rep, &args, pool));
+  SVN_ERR(svn_fs_base__retry_txn(new_args.fs, txn_body_write_rep, &args,
+                                 FALSE, pool));
 
   return SVN_NO_ERROR;
 }
 
 
 static svn_error_t *
-read_rep(const char **msg,
-         svn_boolean_t msg_only,
-         svn_test_opts_t *opts,
+read_rep(const svn_test_opts_t *opts,
          apr_pool_t *pool)
 {
   struct rep_args new_args;
@@ -199,8 +188,8 @@ read_rep(const char **msg,
     (pool, "((fulltext 0  (md5 16 XXXXXXXXXXXXXXXX)) kfogel31337");
   char *new_rep_after = apr_pstrdup
     (pool, "((fulltext 0  (md5 16 XXXXXXXXXXXXXXXX)) a83t2Z0");
-  int rep_after_len = strlen(rep_after);
-  int new_rep_after_len = strlen(new_rep_after);
+  size_t rep_after_len = strlen(rep_after);
+  size_t new_rep_after_len = strlen(new_rep_after);
 
   /* Replace the fake fake checksums with the real fake checksums.
      And someday, when checksums are actually calculated, we can
@@ -217,11 +206,6 @@ read_rep(const char **msg,
         *p = '\0';
   }
 
-  *msg = "write and overwrite a new rep; confirm with reads";
-
-  if (msg_only)
-    return SVN_NO_ERROR;
-
   /* Create a new fs and repos */
   SVN_ERR(svn_test__create_bdb_fs
           (&fs, "test-repo-read-rep", opts,
@@ -234,8 +218,8 @@ read_rep(const char **msg,
   new_args.key = NULL;
 
   /* Write new rep to reps table. */
-  SVN_ERR(svn_fs_base__retry_txn(new_args.fs,
-                                 txn_body_write_new_rep, &new_args, pool));
+  SVN_ERR(svn_fs_base__retry_txn(new_args.fs, txn_body_write_new_rep,
+                                 &new_args, FALSE, pool));
 
   /* Make sure we got a valid key. */
   if (new_args.key == NULL)
@@ -246,8 +230,8 @@ read_rep(const char **msg,
   read_args.fs = new_args.fs;
   read_args.skel = NULL;
   read_args.key = new_args.key;
-  SVN_ERR(svn_fs_base__retry_txn(new_args.fs,
-                                 txn_body_read_rep, &read_args, pool));
+  SVN_ERR(svn_fs_base__retry_txn(new_args.fs, txn_body_read_rep, &read_args,
+                                 FALSE, pool));
 
   /* Make sure the skel matches. */
   if (! read_args.skel)
@@ -265,15 +249,15 @@ read_rep(const char **msg,
   args.key = new_args.key;
 
   /* Overwrite first rep in reps table. */
-  SVN_ERR(svn_fs_base__retry_txn(new_args.fs,
-                                 txn_body_write_rep, &args, pool));
+  SVN_ERR(svn_fs_base__retry_txn(new_args.fs, txn_body_write_rep, &args,
+                                 FALSE, pool));
 
   /* Read the new rep back from the reps table (using the same FS and
      key as the first read...let's make sure this thing didn't get
      written to the wrong place). */
   read_args.skel = NULL;
-  SVN_ERR(svn_fs_base__retry_txn(new_args.fs,
-                                 txn_body_read_rep, &read_args, pool));
+  SVN_ERR(svn_fs_base__retry_txn(new_args.fs, txn_body_read_rep, &read_args,
+                                 FALSE, pool));
 
   /* Make sure the skel matches. */
   if (! read_args.skel)
@@ -290,9 +274,7 @@ read_rep(const char **msg,
 
 
 static svn_error_t *
-delete_rep(const char **msg,
-           svn_boolean_t msg_only,
-           svn_test_opts_t *opts,
+delete_rep(const svn_test_opts_t *opts,
            apr_pool_t *pool)
 {
   struct rep_args new_args;
@@ -302,11 +284,6 @@ delete_rep(const char **msg,
   svn_fs_t *fs;
   svn_error_t *err;
 
-  *msg = "write, then delete, a new rep; confirm deletion";
-
-  if (msg_only)
-    return SVN_NO_ERROR;
-
   /* Create a new fs and repos */
   SVN_ERR(svn_test__create_bdb_fs
           (&fs, "test-repo-delete-rep", opts,
@@ -318,8 +295,8 @@ delete_rep(const char **msg,
   new_args.key = NULL;
 
   /* Write new rep to reps table. */
-  SVN_ERR(svn_fs_base__retry_txn(new_args.fs,
-                                 txn_body_write_new_rep, &new_args, pool));
+  SVN_ERR(svn_fs_base__retry_txn(new_args.fs, txn_body_write_new_rep,
+                                 &new_args, FALSE, pool));
 
   /* Make sure we got a valid key. */
   if (new_args.key == NULL)
@@ -329,15 +306,15 @@ delete_rep(const char **msg,
   /* Delete the rep we just wrote. */
   delete_args.fs = new_args.fs;
   delete_args.key = new_args.key;
-  SVN_ERR(svn_fs_base__retry_txn(new_args.fs,
-                                 txn_body_delete_rep, &delete_args, pool));
+  SVN_ERR(svn_fs_base__retry_txn(new_args.fs, txn_body_delete_rep,
+                                 &delete_args, FALSE, pool));
 
   /* Try to read the new rep back from the reps table. */
   read_args.fs = new_args.fs;
   read_args.skel = NULL;
   read_args.key = new_args.key;
-  err = svn_fs_base__retry_txn(new_args.fs,
-                               txn_body_read_rep, &read_args, pool);
+  err = svn_fs_base__retry_txn(new_args.fs, txn_body_read_rep, &read_args,
+                               FALSE, pool);
 
   /* We better have an error... */
   if ((! err) && (read_args.skel))
@@ -383,7 +360,7 @@ verify_expected_record(svn_fs_t *fs,
                              size, expected_len);
 
   /* Read the string back in 100-byte chunks. */
-  text = svn_stringbuf_create("", trail->pool);
+  text = svn_stringbuf_create_empty(trail->pool);
   while (1)
     {
       size = sizeof(buf);
@@ -528,20 +505,13 @@ static const char *bigstring3 =
 
 
 static svn_error_t *
-test_strings(const char **msg,
-             svn_boolean_t msg_only,
-             svn_test_opts_t *opts,
+test_strings(const svn_test_opts_t *opts,
              apr_pool_t *pool)
 {
   struct string_args args;
   svn_fs_t *fs;
   svn_stringbuf_t *string;
 
-  *msg = "test many strings table functions together";
-
-  if (msg_only)
-    return SVN_NO_ERROR;
-
   /* Create a new fs and repos */
   SVN_ERR(svn_test__create_bdb_fs
           (&fs, "test-repo-test-strings", opts,
@@ -564,8 +534,8 @@ test_strings(const char **msg,
   args.key = NULL;
   args.text = bigstring1;
   args.len = strlen(bigstring1);
-  SVN_ERR(svn_fs_base__retry_txn(args.fs,
-                                 txn_body_string_append, &args, pool));
+  SVN_ERR(svn_fs_base__retry_txn(args.fs, txn_body_string_append, &args,
+                                 FALSE, pool));
 
   /* Make sure a key was returned. */
   if (! args.key)
@@ -573,52 +543,52 @@ test_strings(const char **msg,
                             "write of new string failed to return new key");
 
   /* Verify record's size and contents. */
-  SVN_ERR(svn_fs_base__retry_txn(args.fs,
-                                 txn_body_verify_string, &args, pool));
+  SVN_ERR(svn_fs_base__retry_txn(args.fs, txn_body_verify_string, &args,
+                                 FALSE, pool));
 
   /* Append a second string to our first one. */
   args.text = bigstring2;
   args.len = strlen(bigstring2);
-  SVN_ERR(svn_fs_base__retry_txn(args.fs,
-                                 txn_body_string_append, &args, pool));
+  SVN_ERR(svn_fs_base__retry_txn(args.fs, txn_body_string_append, &args,
+                                 FALSE, pool));
 
   /* Verify record's size and contents. */
   string = svn_stringbuf_create(bigstring1, pool);
   svn_stringbuf_appendcstr(string, bigstring2);
   args.text = string->data;
   args.len = string->len;
-  SVN_ERR(svn_fs_base__retry_txn(args.fs,
-                                 txn_body_verify_string, &args, pool));
+  SVN_ERR(svn_fs_base__retry_txn(args.fs, txn_body_verify_string, &args,
+                                 FALSE, pool));
 
   /* Clear the record */
-  SVN_ERR(svn_fs_base__retry_txn(args.fs,
-                                 txn_body_string_clear, &args, pool));
+  SVN_ERR(svn_fs_base__retry_txn(args.fs, txn_body_string_clear, &args,
+                                 FALSE, pool));
 
   /* Verify record's size and contents. */
   args.text = "";
   args.len = 0;
-  SVN_ERR(svn_fs_base__retry_txn(args.fs,
-                                 txn_body_verify_string, &args, pool));
+  SVN_ERR(svn_fs_base__retry_txn(args.fs, txn_body_verify_string, &args,
+                                 FALSE, pool));
 
   /* Append a third string to our first one. */
   args.text = bigstring3;
   args.len = strlen(bigstring3);
-  SVN_ERR(svn_fs_base__retry_txn(args.fs,
-                                 txn_body_string_append, &args, pool));
+  SVN_ERR(svn_fs_base__retry_txn(args.fs, txn_body_string_append, &args,
+                                 FALSE, pool));
 
   /* Verify record's size and contents. */
-  SVN_ERR(svn_fs_base__retry_txn(args.fs,
-                                 txn_body_verify_string, &args, pool));
+  SVN_ERR(svn_fs_base__retry_txn(args.fs, txn_body_verify_string, &args,
+                                 FALSE, pool));
 
   /* Delete our record...she's served us well. */
-  SVN_ERR(svn_fs_base__retry_txn(args.fs,
-                                 txn_body_string_delete, &args, pool));
+  SVN_ERR(svn_fs_base__retry_txn(args.fs, txn_body_string_delete, &args,
+                                 FALSE, pool));
 
   /* Now, we expect a size request on this record to fail with
      SVN_ERR_FS_NO_SUCH_STRING. */
   {
     svn_error_t *err = svn_fs_base__retry_txn(args.fs, txn_body_string_size,
-                                              &args, pool);
+                                              &args, FALSE, pool);
 
     if (! err)
       return svn_error_create(SVN_ERR_FS_GENERAL, NULL,
@@ -634,19 +604,12 @@ test_strings(const char **msg,
 
 
 static svn_error_t *
-write_null_string(const char **msg,
-                  svn_boolean_t msg_only,
-                  svn_test_opts_t *opts,
+write_null_string(const svn_test_opts_t *opts,
                   apr_pool_t *pool)
 {
   struct string_args args;
   svn_fs_t *fs;
 
-  *msg = "write a null string";
-
-  if (msg_only)
-    return SVN_NO_ERROR;
-
   /* Create a new fs and repos */
   SVN_ERR(svn_test__create_bdb_fs
           (&fs, "test-repo-test-strings", opts,
@@ -657,26 +620,19 @@ write_null_string(const char **msg,
   args.text = NULL;
   args.len = 0;
   SVN_ERR(svn_fs_base__retry_txn(args.fs, txn_body_string_append, &args,
-                                 pool));
+                                 FALSE, pool));
 
   return SVN_NO_ERROR;
 }
 
 
 static svn_error_t *
-abort_string(const char **msg,
-             svn_boolean_t msg_only,
-             svn_test_opts_t *opts,
+abort_string(const svn_test_opts_t *opts,
              apr_pool_t *pool)
 {
   struct string_args args, args2;
   svn_fs_t *fs;
 
-  *msg = "write a string, then abort during an overwrite";
-
-  if (msg_only)
-    return SVN_NO_ERROR;
-
   /* Create a new fs and repos */
   SVN_ERR(svn_test__create_bdb_fs
           (&fs, "test-repo-abort-string", opts,
@@ -694,8 +650,8 @@ abort_string(const char **msg,
   args.key = NULL;
   args.text = bigstring1;
   args.len = strlen(bigstring1);
-  SVN_ERR(svn_fs_base__retry_txn(args.fs,
-                                 txn_body_string_append, &args, pool));
+  SVN_ERR(svn_fs_base__retry_txn(args.fs, txn_body_string_append, &args,
+                                 FALSE, pool));
 
   /* Make sure a key was returned. */
   if (! args.key)
@@ -703,8 +659,8 @@ abort_string(const char **msg,
                             "write of new string failed to return new key");
 
   /* Verify record's size and contents. */
-  SVN_ERR(svn_fs_base__retry_txn(args.fs,
-                                 txn_body_verify_string, &args, pool));
+  SVN_ERR(svn_fs_base__retry_txn(args.fs, txn_body_verify_string, &args,
+                                 FALSE, pool));
 
   /* Append a second string to our first one. */
   args2.fs = fs;
@@ -716,7 +672,7 @@ abort_string(const char **msg,
 
     /* This function is *supposed* to fail with SVN_ERR_TEST_FAILED */
     err = svn_fs_base__retry_txn(args.fs, txn_body_string_append_fail,
-                                 &args2, pool);
+                                 &args2, FALSE, pool);
     if ((! err) || (err->apr_err != SVN_ERR_TEST_FAILED))
       return svn_error_create(SVN_ERR_TEST_FAILED, err,
                               "failed to intentionally abort a trail");
@@ -724,27 +680,20 @@ abort_string(const char **msg,
   }
 
   /* Verify that record's size and contents are still that of string1 */
-  SVN_ERR(svn_fs_base__retry_txn(args.fs,
-                                 txn_body_verify_string, &args, pool));
+  SVN_ERR(svn_fs_base__retry_txn(args.fs, txn_body_verify_string, &args,
+                                 FALSE, pool));
 
   return SVN_NO_ERROR;
 }
 
 static svn_error_t *
-copy_string(const char **msg,
-            svn_boolean_t msg_only,
-            svn_test_opts_t *opts,
+copy_string(const svn_test_opts_t *opts,
             apr_pool_t *pool)
 {
   struct string_args args;
   svn_fs_t *fs;
   const char *old_key;
 
-  *msg = "create and copy a string";
-
-  if (msg_only)
-    return SVN_NO_ERROR;
-
   /* Create a new fs and repos */
   SVN_ERR(svn_test__create_bdb_fs
           (&fs, "test-repo-copy-string", opts,
@@ -755,8 +704,8 @@ copy_string(const char **msg,
   args.key = NULL;
   args.text = bigstring1;
   args.len = strlen(bigstring1);
-  SVN_ERR(svn_fs_base__retry_txn(args.fs,
-                                 txn_body_string_append, &args, pool));
+  SVN_ERR(svn_fs_base__retry_txn(args.fs, txn_body_string_append, &args,
+                                 FALSE, pool));
 
   /* Make sure a key was returned. */
   if (! (old_key = args.key))
@@ -764,8 +713,8 @@ copy_string(const char **msg,
                             "write of new string failed to return new key");
 
   /* Now copy that string into a new location. */
-  SVN_ERR(svn_fs_base__retry_txn(args.fs,
-                                 txn_body_string_copy, &args, pool));
+  SVN_ERR(svn_fs_base__retry_txn(args.fs, txn_body_string_copy, &args,
+                                 FALSE, pool));
 
   /* Make sure a different key was returned. */
   if ((! args.key) || (! strcmp(old_key, args.key)))
@@ -773,8 +722,8 @@ copy_string(const char **msg,
                             "copy of string failed to return new key");
 
   /* Verify record's size and contents. */
-  SVN_ERR(svn_fs_base__retry_txn(args.fs,
-                                 txn_body_verify_string, &args, pool));
+  SVN_ERR(svn_fs_base__retry_txn(args.fs, txn_body_verify_string, &args,
+                                 FALSE, pool));
 
   return SVN_NO_ERROR;
 }
@@ -786,13 +735,21 @@ copy_string(const char **msg,
 struct svn_test_descriptor_t test_funcs[] =
   {
     SVN_TEST_NULL,
-    SVN_TEST_PASS(write_new_rep),
-    SVN_TEST_PASS(write_rep),
-    SVN_TEST_PASS(read_rep),
-    SVN_TEST_PASS(delete_rep),
-    SVN_TEST_PASS(test_strings),
-    SVN_TEST_PASS(write_null_string),
-    SVN_TEST_PASS(abort_string),
-    SVN_TEST_PASS(copy_string),
+    SVN_TEST_OPTS_PASS(write_new_rep,
+                       "write a new rep, get a new key back"),
+    SVN_TEST_OPTS_PASS(write_rep,
+                       "write a new rep, then overwrite it"),
+    SVN_TEST_OPTS_PASS(read_rep,
+                       "write and overwrite a new rep; confirm with reads"),
+    SVN_TEST_OPTS_PASS(delete_rep,
+                       "write, then delete, a new rep; confirm deletion"),
+    SVN_TEST_OPTS_PASS(test_strings,
+                       "test many strings table functions together"),
+    SVN_TEST_OPTS_PASS(write_null_string,
+                       "write a null string"),
+    SVN_TEST_OPTS_PASS(abort_string,
+                       "write a string, then abort during an overwrite"),
+    SVN_TEST_OPTS_PASS(copy_string,
+                       "create and copy a string"),
     SVN_TEST_NULL
   };
